package com.muyu.etl.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.etl.domain.BasicConfigInfo;
import com.muyu.etl.domain.resp.TableTreeResp;
import com.muyu.etl.service.BasicConfigInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 基础信息Controller
 *
 * @author muyu
 * @date 2024-04-21
 */
@RestController
@RequestMapping("/info")
public class BasicConfigInfoController extends BaseController
{
    @Autowired
    private BasicConfigInfoService basicConfigInfoService;

    /**
     * 查询基础信息列表
     */
    @RequiresPermissions("etl:info:list")
    @GetMapping("/list")
    public Result<TableDataInfo<BasicConfigInfo>> list(BasicConfigInfo basicConfigInfo)
    {
        startPage();
        List<BasicConfigInfo> list = basicConfigInfoService.selectBasicConfigInfoList(basicConfigInfo);
        return getDataTable(list);
    }

    /**
     * 导出基础信息列表
     */
    @RequiresPermissions("etl:info:export")
    @Log(title = "基础信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BasicConfigInfo basicConfigInfo)
    {
        List<BasicConfigInfo> list = basicConfigInfoService.selectBasicConfigInfoList(basicConfigInfo);
        ExcelUtil<BasicConfigInfo> util = new ExcelUtil<BasicConfigInfo>(BasicConfigInfo.class);
        util.exportExcel(response, list, "基础信息数据");
    }

    /**
     * 获取基础信息详细信息
     */
    @RequiresPermissions("etl:info:query")
    @GetMapping(value = "/{id}")
    public Result getInfo(@PathVariable("id") Long id)
    {
        return success(basicConfigInfoService.selectBasicConfigInfoById(id));
    }

    /**
     * 新增基础信息
     */
    @RequiresPermissions("etl:info:add")
    @Log(title = "基础信息", businessType = BusinessType.INSERT)
    @PostMapping
    public Result add(@RequestBody BasicConfigInfo configQueryReq)
    {
        return toAjax(basicConfigInfoService.insertBasicConfigInfo(configQueryReq));
    }

    /**
     * 修改基础信息
     */
    @RequiresPermissions("etl:info:edit")
    @Log(title = "基础信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result edit(@RequestBody BasicConfigInfo basicConfigInfo)
    {
        return toAjax(basicConfigInfoService.updateBasicConfigInfo(basicConfigInfo));
    }

    /**
     * 删除基础信息
     */
    @RequiresPermissions("etl:info:remove")
    @Log(title = "基础信息", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public Result remove(@PathVariable Long[] ids)
    {
        return toAjax(basicConfigInfoService.deleteBasicConfigInfoByIds(ids));
    }

    /**
     * 测试连接
     */
    @RequiresPermissions("etl:info:test")
    @Log(title = "测试连接")
    @PostMapping("/connectionTest")
    public Result connectionTest(@RequestBody BasicConfigInfo basicConfigInfo) throws ServletException {
        return toAjax(basicConfigInfoService.connectionTest(basicConfigInfo));
    }

    @RequiresPermissions("etl:info:test")
    @Log(title = "获取成功链接中的")
    @GetMapping("/dataConstruct")
    public Result<TableDataInfo<List>> getData(){
        return getDataTable(basicConfigInfoService.getDataByEtl());
    }

    @RequiresPermissions("etl:table:list")
    @Log(title = "获取已成功链接的树级结构")
    @GetMapping("/getTableTree")
    public Result<List<TableTreeResp>> getTableTree(){
        return Result.success(basicConfigInfoService.getTableTree());
    }
}
