package com.muyu.etl.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.etl.domain.BasicConfigInfo;
import com.muyu.etl.domain.Structure;
import com.muyu.etl.domain.TableInfo;
import com.muyu.etl.domain.resp.TableInfoStructureResp;
import com.muyu.etl.domain.resp.TableTreeResp;
import com.muyu.etl.mapper.BasicConfigInfoMapper;
import com.muyu.etl.service.BasicConfigInfoService;
import com.muyu.etl.service.StructureService;
import com.muyu.etl.service.TableInfoService;
import lombok.SneakyThrows;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.ServletException;
import java.sql.*;
import java.util.Date;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Service业务层处理
 *
 * @author muyu
 * @date 2024-04-20
 */
@Service
@Log4j2
public class BasicConfigInfoServiceImpl  extends ServiceImpl<BasicConfigInfoMapper, BasicConfigInfo> implements BasicConfigInfoService
{
    @Autowired
    private BasicConfigInfoMapper basicConfigInfoMapper;

    @Autowired
    private StructureService structureService;

    @Autowired
    private TableInfoService tableInfoService;


    /**
     * 查询
     *
     * @param id 主键
     * @return
     */
    @Override
    public BasicConfigInfo selectBasicConfigInfoById(Long id)
    {
        return basicConfigInfoMapper.selectBasicConfigInfoById(id);
    }

    /**
     * 查询列表
     *
     * @param basicConfigInfo
     * @return
     */
    @Override
    public List<BasicConfigInfo> selectBasicConfigInfoList(BasicConfigInfo basicConfigInfo)
    {
        return basicConfigInfoMapper.selectBasicConfigInfoList(basicConfigInfo);
    }

    /**
     * 新增
     *
     * @param configQueryReq
     * @return 结果
     */
    @Override
    public int insertBasicConfigInfo(BasicConfigInfo configQueryReq)
    {
        return basicConfigInfoMapper.insertBasicConfigInfo(configQueryReq);
    }

    /**
     * 修改
     *
     * @param configQueryReq
     * @return 结果
     */
    @Override
    public int updateBasicConfigInfo(BasicConfigInfo configQueryReq)
    {
        return basicConfigInfoMapper.updateBasicConfigInfo(configQueryReq);
    }

    /**
     * 批量删除
     *
     * @param ids 需要删除的主键
     * @return 结果
     */
    @Override
    public int deleteBasicConfigInfoByIds(Long[] ids)
    {
        return basicConfigInfoMapper.deleteBasicConfigInfoByIds(ids);
    }

    /**
     * 删除信息
     *
     * @param id 主键
     * @return 结果
     */
    @Override
    public int deleteBasicConfigInfoById(Long id)
    {
        return basicConfigInfoMapper.deleteBasicConfigInfoById(id);
    }

    @Override
    public boolean connectionTest(BasicConfigInfo basicConfigInfo) throws ServletException {
        //定义下面需要的对象
        String host = basicConfigInfo.getHost();
        String port = basicConfigInfo.getPort();
        String databaseName = basicConfigInfo.getDatabaseName();
        String databaseType = basicConfigInfo.getDatabaseType();

        String url = "jdbc:"+databaseType+"://"+host+":"+port+"/"+databaseName+"?"+basicConfigInfo.getConnectionParams();
        String user = basicConfigInfo.getUsername();
        String password = basicConfigInfo.getPassword();
        Connection conn= null;
        try {
            conn = DriverManager.getConnection(url, user, password);
            System.out.println("Connected to the MySQL server successfully.");
            //同步数据库信息
            //树级结构，库，表
            TableInfo tableInfo = TableInfo.builder()
                .basicId(basicConfigInfo.getId())
                .parentId(0L)
                .tableRemark("")
                    .center("Y")
                .tableName(basicConfigInfo.getDataSourcesSystemName() + "(" + databaseName + ")")
                .createBy(SecurityUtils.getUsername())
                .createTime(new Date())
            .build();
            tableInfoService.saveOrUpdate(tableInfo,new LambdaUpdateWrapper<TableInfo>(TableInfo.class){{
                eq(TableInfo::getTableName,tableInfo.getTableName());
                eq(TableInfo::getBasicId,basicConfigInfo.getId());
            }});
            DatabaseMetaData metaData = conn.getMetaData();
            ResultSet rs = metaData.getTables(databaseName,
                    null, "%", new String[]{"TABLE", "VIEW"});
                while (rs.next()) {
                    //表名
                    String tableName1 = rs.getString("TABLE_NAME");
                    String tableRemark = rs.getString("REMARKS");
                    Connection finalConn = conn;
                    PreparedStatement ps = conn.prepareStatement("select* from " + tableName1);
                    ResultSet rset = ps.executeQuery();
                    Long rowCount = 0L;
                    while(rset.next()) {rowCount++;}
                    TableInfo build = TableInfo.builder()
                            .basicId(basicConfigInfo.getId())
                            .tableName(tableName1)
                            .tableRemark(tableRemark.isEmpty()? "null":tableRemark)
                            .parentId(tableInfo.getId())
                            .center("Y")
                            .updateBy(SecurityUtils.getUsername())
                            .dataNum(rowCount)
                            .updateTime(new Date())
                            .build();
                    tableInfoService.saveOrUpdate(build,new LambdaUpdateWrapper<TableInfo>(TableInfo.class){{
                        eq(TableInfo::getTableName,build.getTableName());
                        eq(TableInfo::getBasicId,basicConfigInfo.getId());
                    }});
                    TableInfo table = tableInfoService.selectTableInfoByName(build);
                    Runnable thread = new Runnable() {
                        @SneakyThrows
                        @Override
                        public void run() {
                            try {
                                syncData(finalConn, databaseName, table);
                            } catch (SQLException e) {
                                throw new ServletException("连接失败");
                            }
                        }
                    };
                    thread.run();
                }
        } catch (SQLException e) {
            throw new ServletException("连接失败");
        }
        return true;
    }

    public void syncData (Connection conn, String databaseName, TableInfo table) throws SQLException {
        PreparedStatement ps = conn.prepareStatement(
                    "    SELECT  " +
                        "    COLUMN_NAME AS '字段', " +
                        "    COLUMN_COMMENT AS '字段注释', " +
                        "    CASE WHEN COLUMN_KEY = 'PRI' THEN '是' ELSE '否' END AS '是否主键'," +
                        "    CASE \n" +
                        "    WHEN DATA_TYPE = 'int' THEN 'Integer' " +
                        "    WHEN DATA_TYPE = 'bigint' THEN 'Long' " +
                        "    WHEN DATA_TYPE = 'varchar' THEN 'String' " +
                        "    WHEN DATA_TYPE = 'decimal' THEN 'BigDecimal' " +
                        "    WHEN DATA_TYPE = 'tinyint' AND COLUMN_TYPE = 'tinyint(1)' THEN 'Boolean'" +
                        "    ELSE DATA_TYPE -- 如果无法映射，则返回原始数据库类型  \n" +
                        "    END AS 'Java类型', " +
                        "    DATA_TYPE AS '数据库类型', -- 原始的数据库类型  \n" +
                        "    COLUMN_TYPE AS '详细的数据库类型', -- 更详细的数据库类型，可能包含长度、精度等  \n" +
                        "    CHARACTER_MAXIMUM_LENGTH AS '长度',  \n" +
                        "    NUMERIC_SCALE AS '小数位',  \n" +
                        "    IS_NULLABLE AS '是否为空',  \n" +
                        "    COLUMN_DEFAULT AS '默认值'  \n" +
                        "FROM  INFORMATION_SCHEMA.COLUMNS  WHERE  \n" +
                        "TABLE_SCHEMA = '" + databaseName + "' -- 替换为你的数据库名称  \n" +
                        "AND TABLE_NAME = '" + table.getTableName() + "'");
//        "SELECT COLUMN_NAME,DATA_TYPE,IS_NULLABLE,COLUMN_KEY,COLUMN_DEFAULT,
//        COLUMN_COMMENT,CHARACTER_MAXIMUM_LENGTH,NUMERIC_PRECISION,
//        NUMERIC_SCALE FROM INFORMATION_SCHEMA.COLUMNS
//        WHERE TABLE_SCHEMA = '数据库名' AND TABLE_NAME = '表名'"
        ResultSet resultSet = ps.executeQuery();
        while (resultSet.next()){
            String columnName = String.valueOf( resultSet.getString(1));
            String columnComment = String.valueOf( resultSet.getObject(2));
            String columnKey = String.valueOf( resultSet.getObject(3));
            String end = String.valueOf( resultSet.getObject(4));
            String dataType = String.valueOf( resultSet.getObject(5));
            String columnType = String.valueOf( resultSet.getObject(6));
            String characterMaximumLength = String.valueOf(resultSet.getInt(7));
            String NumericScale = String.valueOf(resultSet.getInt(8));
            String isNullable = String.valueOf( resultSet.getObject(9));
            String columnDefault = String.valueOf( resultSet.getObject(10));
            Structure build = Structure.builder()
                    .tableId(table.getId())
                    .columnName(String.valueOf(columnName))
                    .columnRemark(columnComment)
                    .isPrimary("是".equals(columnKey) ? "Y" : "N")
                    .javaType( end)
                    .columnType( dataType)
                    .columnType(columnType)
                    .columnLength(characterMaximumLength)
                    .columnDecimals( NumericScale)
                    .isNull("YES".equals(isNullable) ? "Y" : "N")
                    .defaultValue( columnDefault)
                    .build();
            structureService.saveOrUpdate(build,new LambdaUpdateWrapper<Structure>(){{
                eq(Structure::getTableId,build.getTableId());
                eq(Structure::getColumnName,build.getColumnName());
                eq(Structure::getColumnRemark,build.getColumnRemark());
            }});
        }
    }

    /**
     * 获取数据接入接口数据
     * @return
     */
    @Override
    public List getDataByEtl() {
        List<BasicConfigInfo> list = this.list();
        List<Map<String,List<String>>> mapList = new ArrayList<>();
        for (BasicConfigInfo info : list) {
            //定义下面需要的对象
            String url = "jdbc:" + info.getDatabaseType() + "://" + info.getHost() + ":" + info.getPort() + "/" + info.getDatabaseName() + "";
            String user = info.getUsername();
            String password = info.getPassword();
            Connection conn = null;
            try {
                conn = DriverManager.getConnection(url, user, password);
                DatabaseMetaData metaData = conn.getMetaData();
                ResultSet resultSet = metaData.getCatalogs();
                while (resultSet.next()){
                    //库名
                    String catalogs = resultSet.getString("TABLE_CAT");
                    log.info(catalogs);
                    // 获取表名
                    ResultSet rs = metaData.getTables(info.getDatabaseName(),null, "%", new String[]{"TABLE", "VIEW"});
                        try {
                            ArrayList<String> tableName = new ArrayList<>();
                            while (rs.next()) {
                                //表名
                                String tableName1 = rs.getString("TABLE_NAME");
                                tableName.add( rs.getString("TABLE_NAME"));
                            }
                            mapList.add(new HashMap<>(){{
                                //库名，表名
                                put(catalogs,tableName);
                            }});
                        }catch (Exception exception){
                            continue;
                        }
                }
            } catch (Exception e){
                log.error(e.getMessage());
                continue;
            }
        }
        return mapList;
    }

    /**
     * 获取响应体对象
     * @return
     */

    @Override
    public List<TableTreeResp> getTableTree() {
        List<TableTreeResp> tableTreeRespList = tableInfoService.selectTableInfoList(new TableInfo(){{setParentId(0L);}}).stream().map(tableInfo ->{
                    BasicConfigInfo basicConfigInfo = this.selectBasicConfigInfoById(tableInfo.getBasicId());
                    List<TableInfoStructureResp> tableInfoStructureRespList = tableInfoService.selectTableInfoList(new TableInfo() {{
                        setParentId(tableInfo.getId());
                    }}).stream().map(info -> {
                        List<Structure> structureList = structureService.list(new LambdaQueryWrapper<Structure>().eq(Structure::getTableId,info.getId()));
                        return TableInfoStructureResp.builder()
                                .id(info.getId())
                                .tableName(info.getTableName())
                                .center(info.getCenter())
                                .tableRemark(info.getTableRemark())
                                .dataNum(info.getDataNum())
                                .parentId(info.getParentId())
                                .databaseType(basicConfigInfo.getDatabaseType())
                                .structureList(structureList)
                                .basicId(info.getBasicId())
                                .build();
                    }).collect(Collectors.toList());
                    return TableTreeResp.builder()
                            .tableInfo(tableInfo)
                            .basicConfigInfo(basicConfigInfo)
                            .Children(tableInfoStructureRespList)
                            .build();
                }).collect(Collectors.toList());
        return tableTreeRespList;
    }

}
